#include <errno.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>

#include <proc_service.h>
#include <thread_db.h>

static void handle_signal (int signum)
{
	signal (signum, SIG_DFL);
	psignal (signum, "pstack signal received");
//	if (thePid)
//		detachall();
	exit (1);
}

void usage(const char *argv0, const char *param)
{
	fprintf(stderr, "Invalid parameter '%s'.\n", param);
	fprintf(stderr, "Usage: %s <pid> [one or more]\n", argv0);
	exit(1);
}

int main(int argc, char **argv)
{
	int i;
	const char *argv0 = argv[0];

	/*
	 * Arrange to detach if we get an unexpected signal.  This prevents
	 * threads from being left in a suspended state if (for example) we
	 * try to get a stack trace from a threaded process which has
	 * been stripped.
	 */
	for (i = 0; i < NSIG; i++)
		if (i != SIGCHLD)
			signal (i, handle_signal);

	for (argc--, argv++; argc > 0; argc--, argv++) {
		char *endptr = NULL;
		struct ps_prochandle *ph;
		td_thragent_t *th;
		int thePid;

		thePid = strtol(*argv, &endptr, 0);
		if (!*argv || *endptr || errno==ERANGE)
			usage(argv0, *argv);
		if (!thePid || thePid == getpid()) {
			fprintf(stderr, "Invalid PID %d\n", thePid);
			continue;
		}

		ps_init();

		// grab the task
		ph = ps_attach(thePid);
		if (!ph) {
			fprintf(stderr, "Couldn't grab task\n");
			exit(-1);
		}

		// get auxval

		// init thread debug
/*		td_init();
		do {
			int i;
			char **symbols = td_symbol_list();
			for (i=0; symbols[i];i++)
				printf("%s\n", symbols[i]);
		} while (0);
*/
		if (td_init() != TD_OK || td_ta_new(ph, &th) != TD_OK) {
			fprintf(stderr, "Not threaded\n");
		} else {
			td_ta_delete(th);
		}

		ps_release(ph);

		ps_exit();
	}

	return 0;
}
